use num_traits::FromPrimitive;
use serde::{Deserialize, Serialize};

use super::MessageType;

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Header([u8; Self::HEADER_LENGTH as usize]);

impl Header {
    pub(crate) const HEADER_LENGTH: u8 = 10;

    pub fn new(
        body_length: u8,
        device_type: u8,
        device_protocol_version: u8,
        message_type: MessageType,
    ) -> Self {
        let length = Self::HEADER_LENGTH + body_length;

        Self([
            // flag
            0xAA,
            // length
            length,
            // device type
            device_type,
            // frame checksum
            0x00, // self._device_type ^ length,
            // unused
            0x00,
            0x00,
            // frame ID
            0x00,
            // frame protocol version
            0x00,
            // device protocol version
            device_protocol_version,
            // frame type
            message_type as u8,
        ])
    }

    pub fn device_protocol_version(&self) -> u8 {
        self.0[8]
    }

    pub fn message_type(&self) -> MessageType {
        MessageType::from_u8(self.0[9]).unwrap()
    }

    pub fn device_type(&self) -> u8 {
        self.0[2]
    }

    pub(crate) fn raw(&self) -> &[u8; Self::HEADER_LENGTH as usize] {
        &self.0
    }
}

impl From<&[u8]> for Header {
    fn from(value: &[u8]) -> Self {
        Self(
            value[0..Self::HEADER_LENGTH as usize]
                .iter()
                .cloned()
                .collect::<Vec<u8>>()
                .try_into()
                .unwrap(),
        )
    }
}
